{-# LANGUAGE MultiParamTypeClasses, FunctionalDependencies #-}
{-# OPTIONS_GHC -fno-warn-duplicate-exports #-}
module Data.FpML.V53.Enum
  ( module Data.FpML.V53.Enum
  ) where
 
import Text.XML.HaXml.Schema.Schema (SchemaType(..),SimpleType(..),Extension(..),Restricts(..))
import Text.XML.HaXml.Schema.Schema as Schema
import qualified Text.XML.HaXml.Schema.PrimitiveTypes as Xsd
 
-- | The type of averaging used in an Asian option. 
data AveragingInOutEnum
instance Eq AveragingInOutEnum
instance Show AveragingInOutEnum
instance Enum AveragingInOutEnum
instance SchemaType AveragingInOutEnum
instance SimpleType AveragingInOutEnum
 
-- | The method of calculation to be used when averaging rates. 
--   Per ISDA 2000 Definitions, Section 6.2. Certain Definitions 
--   Relating to Floating Amounts. 
data AveragingMethodEnum
instance Eq AveragingMethodEnum
instance Show AveragingMethodEnum
instance Enum AveragingMethodEnum
instance SchemaType AveragingMethodEnum
instance SimpleType AveragingMethodEnum
 
-- | When breakage cost is applicable, defines who is 
--   calculating it. 
data BreakageCostEnum
instance Eq BreakageCostEnum
instance Show BreakageCostEnum
instance Enum BreakageCostEnum
instance SchemaType BreakageCostEnum
instance SimpleType BreakageCostEnum
 
-- | Defines which type of bullion is applicable for a Bullion 
--   Transaction. 
data BullionTypeEnum
instance Eq BullionTypeEnum
instance Show BullionTypeEnum
instance Enum BullionTypeEnum
instance SchemaType BullionTypeEnum
instance SimpleType BullionTypeEnum
 
-- | The convention for adjusting any relevant date if it would 
--   otherwise fall on a day that is not a valid business day. 
--   Note that FRN is included here as a type of business day 
--   convention although it does not strictly fall within ISDA's 
--   definition of a Business Day Convention and does not 
--   conform to the simple definition given above. 
data BusinessDayConventionEnum
instance Eq BusinessDayConventionEnum
instance Show BusinessDayConventionEnum
instance Enum BusinessDayConventionEnum
instance SchemaType BusinessDayConventionEnum
instance SimpleType BusinessDayConventionEnum
 
-- | Shows how the transaction is to be settled when it is 
--   exercised. 
data CashPhysicalEnum
instance Eq CashPhysicalEnum
instance Show CashPhysicalEnum
instance Enum CashPhysicalEnum
instance SchemaType CashPhysicalEnum
instance SimpleType CashPhysicalEnum
 
-- | The specification of how a calculation agent will be 
--   determined. 
data CalculationAgentPartyEnum
instance Eq CalculationAgentPartyEnum
instance Show CalculationAgentPartyEnum
instance Enum CalculationAgentPartyEnum
instance SchemaType CalculationAgentPartyEnum
instance SimpleType CalculationAgentPartyEnum
 
-- | The unit in which a commission is denominated. 
data CommissionDenominationEnum
instance Eq CommissionDenominationEnum
instance Show CommissionDenominationEnum
instance Enum CommissionDenominationEnum
instance SchemaType CommissionDenominationEnum
instance SimpleType CommissionDenominationEnum
 
-- | The consequences of Bullion Settlement Disruption Events. 
data CommodityBullionSettlementDisruptionEnum
instance Eq CommodityBullionSettlementDisruptionEnum
instance Show CommodityBullionSettlementDisruptionEnum
instance Enum CommodityBullionSettlementDisruptionEnum
instance SchemaType CommodityBullionSettlementDisruptionEnum
instance SimpleType CommodityBullionSettlementDisruptionEnum
 
-- | A day type classification used in counting the number of 
--   days between two dates for a commodity transaction. 
data CommodityDayTypeEnum
instance Eq CommodityDayTypeEnum
instance Show CommodityDayTypeEnum
instance Enum CommodityDayTypeEnum
instance SchemaType CommodityDayTypeEnum
instance SimpleType CommodityDayTypeEnum
 
-- | The compounding calculation method 
data CompoundingMethodEnum
instance Eq CompoundingMethodEnum
instance Show CompoundingMethodEnum
instance Enum CompoundingMethodEnum
instance SchemaType CompoundingMethodEnum
instance SimpleType CompoundingMethodEnum
 
-- | A day of the seven-day week. 
data DayOfWeekEnum
instance Eq DayOfWeekEnum
instance Show DayOfWeekEnum
instance Enum DayOfWeekEnum
instance SchemaType DayOfWeekEnum
instance SimpleType DayOfWeekEnum
 
-- | A day type classification used in counting the number of 
--   days between two dates. 
data DayTypeEnum
instance Eq DayTypeEnum
instance Show DayTypeEnum
instance Enum DayTypeEnum
instance SchemaType DayTypeEnum
instance SimpleType DayTypeEnum
 
data DealtCurrencyEnum
instance Eq DealtCurrencyEnum
instance Show DealtCurrencyEnum
instance Enum DealtCurrencyEnum
instance SchemaType DealtCurrencyEnum
instance SimpleType DealtCurrencyEnum
 
-- | In respect of a Transaction and a Commodity Reference 
--   Price, the relevant date or month for delivery of the 
--   underlying Commodity. 
data DeliveryDatesEnum
instance Eq DeliveryDatesEnum
instance Show DeliveryDatesEnum
instance Enum DeliveryDatesEnum
instance SchemaType DeliveryDatesEnum
instance SimpleType DeliveryDatesEnum
 
data DeliveryTypeEnum
instance Eq DeliveryTypeEnum
instance Show DeliveryTypeEnum
instance Enum DeliveryTypeEnum
instance SchemaType DeliveryTypeEnum
instance SimpleType DeliveryTypeEnum
 
-- | The ISDA defined value indicating the severity of a 
--   difference. 
data DifferenceSeverityEnum
instance Eq DifferenceSeverityEnum
instance Show DifferenceSeverityEnum
instance Enum DifferenceSeverityEnum
instance SchemaType DifferenceSeverityEnum
instance SimpleType DifferenceSeverityEnum
 
-- | The ISDA defined value indicating the nature of a 
--   difference. 
data DifferenceTypeEnum
instance Eq DifferenceTypeEnum
instance Show DifferenceTypeEnum
instance Enum DifferenceTypeEnum
instance SchemaType DifferenceTypeEnum
instance SimpleType DifferenceTypeEnum
 
-- | The method of calculating discounted payment amounts 
data DiscountingTypeEnum
instance Eq DiscountingTypeEnum
instance Show DiscountingTypeEnum
instance Enum DiscountingTypeEnum
instance SchemaType DiscountingTypeEnum
instance SimpleType DiscountingTypeEnum
 
-- | The specification of how disruption fallbacks will be 
--   represented. 
data DisruptionFallbacksEnum
instance Eq DisruptionFallbacksEnum
instance Show DisruptionFallbacksEnum
instance Enum DisruptionFallbacksEnum
instance SchemaType DisruptionFallbacksEnum
instance SimpleType DisruptionFallbacksEnum
 
-- | Refers to one on the 3 Amounts 
data DividendAmountTypeEnum
instance Eq DividendAmountTypeEnum
instance Show DividendAmountTypeEnum
instance Enum DividendAmountTypeEnum
instance SchemaType DividendAmountTypeEnum
instance SimpleType DividendAmountTypeEnum
 
-- | Defines how the composition of dividends is to be 
--   determined. 
data DividendCompositionEnum
instance Eq DividendCompositionEnum
instance Show DividendCompositionEnum
instance Enum DividendCompositionEnum
instance SchemaType DividendCompositionEnum
instance SimpleType DividendCompositionEnum
 
-- | The reference to a dividend date. 
data DividendDateReferenceEnum
instance Eq DividendDateReferenceEnum
instance Show DividendDateReferenceEnum
instance Enum DividendDateReferenceEnum
instance SchemaType DividendDateReferenceEnum
instance SimpleType DividendDateReferenceEnum
 
-- | The date on which the receiver of the equity return is 
--   entitled to the dividend. 
data DividendEntitlementEnum
instance Eq DividendEntitlementEnum
instance Show DividendEntitlementEnum
instance Enum DividendEntitlementEnum
instance SchemaType DividendEntitlementEnum
instance SimpleType DividendEntitlementEnum
 
-- | Defines the First Period or the Second Period, as specified 
--   in the 2002 ISDA Equity Derivatives Definitions. 
data DividendPeriodEnum
instance Eq DividendPeriodEnum
instance Show DividendPeriodEnum
instance Enum DividendPeriodEnum
instance SchemaType DividendPeriodEnum
instance SimpleType DividendPeriodEnum
 
-- | A type which permits the Dual Currency strike quote basis 
--   to be expressed in terms of the deposit and alternate 
--   currencies. 
data DualCurrencyStrikeQuoteBasisEnum
instance Eq DualCurrencyStrikeQuoteBasisEnum
instance Show DualCurrencyStrikeQuoteBasisEnum
instance Enum DualCurrencyStrikeQuoteBasisEnum
instance SchemaType DualCurrencyStrikeQuoteBasisEnum
instance SimpleType DualCurrencyStrikeQuoteBasisEnum
 
-- | The type of electricity product. 
data ElectricityProductTypeEnum
instance Eq ElectricityProductTypeEnum
instance Show ElectricityProductTypeEnum
instance Enum ElectricityProductTypeEnum
instance SchemaType ElectricityProductTypeEnum
instance SimpleType ElectricityProductTypeEnum
 
-- | Specifies an additional Forward type. 
data EquityOptionTypeEnum
instance Eq EquityOptionTypeEnum
instance Show EquityOptionTypeEnum
instance Enum EquityOptionTypeEnum
instance SchemaType EquityOptionTypeEnum
instance SimpleType EquityOptionTypeEnum
 
-- | The specification of how an OTC option will be exercised. 
data ExerciseStyleEnum
instance Eq ExerciseStyleEnum
instance Show ExerciseStyleEnum
instance Enum ExerciseStyleEnum
instance SchemaType ExerciseStyleEnum
instance SimpleType ExerciseStyleEnum
 
-- | Defines the fee type. 
data FeeElectionEnum
instance Eq FeeElectionEnum
instance Show FeeElectionEnum
instance Enum FeeElectionEnum
instance SchemaType FeeElectionEnum
instance SimpleType FeeElectionEnum
 
-- | The method by which the Flat Rate is calculated for a 
--   commodity freight transaction. 
data FlatRateEnum
instance Eq FlatRateEnum
instance Show FlatRateEnum
instance Enum FlatRateEnum
instance SchemaType FlatRateEnum
instance SimpleType FlatRateEnum
 
-- | Specifies the fallback provisions in respect to the 
--   applicable Futures Price Valuation. 
data FPVFinalPriceElectionFallbackEnum
instance Eq FPVFinalPriceElectionFallbackEnum
instance Show FPVFinalPriceElectionFallbackEnum
instance Enum FPVFinalPriceElectionFallbackEnum
instance SchemaType FPVFinalPriceElectionFallbackEnum
instance SimpleType FPVFinalPriceElectionFallbackEnum
 
-- | The method of FRA discounting, if any, that will apply. 
data FraDiscountingEnum
instance Eq FraDiscountingEnum
instance Show FraDiscountingEnum
instance Enum FraDiscountingEnum
instance SchemaType FraDiscountingEnum
instance SimpleType FraDiscountingEnum
 
-- | The schedule frequency type 
data FrequencyTypeEnum
instance Eq FrequencyTypeEnum
instance Show FrequencyTypeEnum
instance Enum FrequencyTypeEnum
instance SchemaType FrequencyTypeEnum
instance SimpleType FrequencyTypeEnum
 
-- | The specification of whether a barrier within an FX OTC 
--   option is a knockin or knockout, as well as whether it is a 
--   standard barrier or a reverse barrier. 
data FxBarrierTypeEnum
instance Eq FxBarrierTypeEnum
instance Show FxBarrierTypeEnum
instance Enum FxBarrierTypeEnum
instance SchemaType FxBarrierTypeEnum
instance SimpleType FxBarrierTypeEnum
 
-- | The specification of a time period containing values such 
--   as Today, Tomorrow etc. 
data FxTenorPeriodEnum
instance Eq FxTenorPeriodEnum
instance Show FxTenorPeriodEnum
instance Enum FxTenorPeriodEnum
instance SchemaType FxTenorPeriodEnum
instance SimpleType FxTenorPeriodEnum
 
-- | The type of gas product. 
data GasProductTypeEnum
instance Eq GasProductTypeEnum
instance Show GasProductTypeEnum
instance Enum GasProductTypeEnum
instance SchemaType GasProductTypeEnum
instance SimpleType GasProductTypeEnum
 
-- | The type of independent amount convention. 
data IndependentAmountConventionEnum
instance Eq IndependentAmountConventionEnum
instance Show IndependentAmountConventionEnum
instance Enum IndependentAmountConventionEnum
instance SchemaType IndependentAmountConventionEnum
instance SimpleType IndependentAmountConventionEnum
 
-- | The specification of the consequences of Index Events. 
data IndexEventConsequenceEnum
instance Eq IndexEventConsequenceEnum
instance Show IndexEventConsequenceEnum
instance Enum IndexEventConsequenceEnum
instance SchemaType IndexEventConsequenceEnum
instance SimpleType IndexEventConsequenceEnum
 
-- | &gt;Defines whether agent bank is making an interest 
--   payment based on the lender pro-rata share at the end of 
--   the period or based on the lender position throughout the 
--   period. Agent Banks decide which way to calculate the 
--   interest for a deal. 
data InterestCalculationMethodEnum
instance Eq InterestCalculationMethodEnum
instance Show InterestCalculationMethodEnum
instance Enum InterestCalculationMethodEnum
instance SchemaType InterestCalculationMethodEnum
instance SimpleType InterestCalculationMethodEnum
 
-- | The type of calculation. 
data InterestCalculationTypeEnum
instance Eq InterestCalculationTypeEnum
instance Show InterestCalculationTypeEnum
instance Enum InterestCalculationTypeEnum
instance SchemaType InterestCalculationTypeEnum
instance SimpleType InterestCalculationTypeEnum
 
-- | The type of method. 
data InterestMethodEnum
instance Eq InterestMethodEnum
instance Show InterestMethodEnum
instance Enum InterestMethodEnum
instance SchemaType InterestMethodEnum
instance SimpleType InterestMethodEnum
 
-- | The specification of the interest shortfall cap, applicable 
--   to mortgage derivatives. 
data InterestShortfallCapEnum
instance Eq InterestShortfallCapEnum
instance Show InterestShortfallCapEnum
instance Enum InterestShortfallCapEnum
instance SchemaType InterestShortfallCapEnum
instance SimpleType InterestShortfallCapEnum
 
-- | Defines applicable periods for interpolation. 
data InterpolationPeriodEnum
instance Eq InterpolationPeriodEnum
instance Show InterpolationPeriodEnum
instance Enum InterpolationPeriodEnum
instance SchemaType InterpolationPeriodEnum
instance SimpleType InterpolationPeriodEnum
 
-- | Used for indicating the length unit in the Resource type. 
data LengthUnitEnum
instance Eq LengthUnitEnum
instance Show LengthUnitEnum
instance Enum LengthUnitEnum
instance SchemaType LengthUnitEnum
instance SimpleType LengthUnitEnum
 
-- | The specification of how market disruption events will be 
--   represented. 
data MarketDisruptionEventsEnum
instance Eq MarketDisruptionEventsEnum
instance Show MarketDisruptionEventsEnum
instance Enum MarketDisruptionEventsEnum
instance SchemaType MarketDisruptionEventsEnum
instance SimpleType MarketDisruptionEventsEnum
 
-- | The type of mark to market convention. 
data MarkToMarketConventionEnum
instance Eq MarkToMarketConventionEnum
instance Show MarkToMarketConventionEnum
instance Enum MarkToMarketConventionEnum
instance SchemaType MarkToMarketConventionEnum
instance SimpleType MarkToMarketConventionEnum
 
-- | Defines how adjustments will be made to the contract should 
--   one or more of the extraordinary events occur. 
data MethodOfAdjustmentEnum
instance Eq MethodOfAdjustmentEnum
instance Show MethodOfAdjustmentEnum
instance Enum MethodOfAdjustmentEnum
instance SchemaType MethodOfAdjustmentEnum
instance SimpleType MethodOfAdjustmentEnum
 
-- | Defines the consequences of nationalisation, insolvency and 
--   delisting events relating to the underlying. 
data NationalisationOrInsolvencyOrDelistingEventEnum
instance Eq NationalisationOrInsolvencyOrDelistingEventEnum
instance Show NationalisationOrInsolvencyOrDelistingEventEnum
instance Enum NationalisationOrInsolvencyOrDelistingEventEnum
instance SchemaType NationalisationOrInsolvencyOrDelistingEventEnum
instance SimpleType NationalisationOrInsolvencyOrDelistingEventEnum
 
-- | The method of calculating payment obligations when a 
--   floating rate is negative (either due to a quoted negative 
--   floating rate or by operation of a spread that is 
--   subtracted from the floating rate). 
data NegativeInterestRateTreatmentEnum
instance Eq NegativeInterestRateTreatmentEnum
instance Show NegativeInterestRateTreatmentEnum
instance Enum NegativeInterestRateTreatmentEnum
instance SchemaType NegativeInterestRateTreatmentEnum
instance SimpleType NegativeInterestRateTreatmentEnum
 
-- | Defines treatment of non-cash dividends. 
data NonCashDividendTreatmentEnum
instance Eq NonCashDividendTreatmentEnum
instance Show NonCashDividendTreatmentEnum
instance Enum NonCashDividendTreatmentEnum
instance SchemaType NonCashDividendTreatmentEnum
instance SimpleType NonCashDividendTreatmentEnum
 
-- | The conditions that govern the adjustment to the number of 
--   units of the equity swap. 
data NotionalAdjustmentEnum
instance Eq NotionalAdjustmentEnum
instance Show NotionalAdjustmentEnum
instance Enum NotionalAdjustmentEnum
instance SchemaType NotionalAdjustmentEnum
instance SimpleType NotionalAdjustmentEnum
 
-- | Used in both the obligations and deliverable obligations of 
--   the credit default swap to represent a class or type of 
--   securities which apply. 
data ObligationCategoryEnum
instance Eq ObligationCategoryEnum
instance Show ObligationCategoryEnum
instance Enum ObligationCategoryEnum
instance SchemaType ObligationCategoryEnum
instance SimpleType ObligationCategoryEnum
 
-- | Specifies the type of the option. 
data OptionTypeEnum
instance Eq OptionTypeEnum
instance Show OptionTypeEnum
instance Enum OptionTypeEnum
instance SchemaType OptionTypeEnum
instance SimpleType OptionTypeEnum
 
-- | The specification of an interest rate stream payer or 
--   receiver party. 
data PayerReceiverEnum
instance Eq PayerReceiverEnum
instance Show PayerReceiverEnum
instance Enum PayerReceiverEnum
instance SchemaType PayerReceiverEnum
instance SimpleType PayerReceiverEnum
 
-- | The specification of how an FX OTC option with a trigger 
--   payout will be paid if the trigger condition is met. The 
--   contract will specify whether the payout will occur 
--   immediately or on the original value date of the option. 
data PayoutEnum
instance Eq PayoutEnum
instance Show PayoutEnum
instance Enum PayoutEnum
instance SchemaType PayoutEnum
instance SimpleType PayoutEnum
 
-- | The specification of whether payments occur relative to the 
--   calculation period start or end date, or the reset date. 
data PayRelativeToEnum
instance Eq PayRelativeToEnum
instance Show PayRelativeToEnum
instance Enum PayRelativeToEnum
instance SchemaType PayRelativeToEnum
instance SimpleType PayRelativeToEnum
 
-- | The specification of a time period 
data PeriodEnum
instance Eq PeriodEnum
instance Show PeriodEnum
instance Enum PeriodEnum
instance SchemaType PeriodEnum
instance SimpleType PeriodEnum
 
-- | The specification of a time period containing additional 
--   values such as Term. 
data PeriodExtendedEnum
instance Eq PeriodExtendedEnum
instance Show PeriodExtendedEnum
instance Enum PeriodExtendedEnum
instance SchemaType PeriodExtendedEnum
instance SimpleType PeriodExtendedEnum
 
-- | A type used to report how a position originated. 
data PositionOriginEnum
instance Eq PositionOriginEnum
instance Show PositionOriginEnum
instance Enum PositionOriginEnum
instance SchemaType PositionOriginEnum
instance SimpleType PositionOriginEnum
 
data PositionStatusEnum
instance Eq PositionStatusEnum
instance Show PositionStatusEnum
instance Enum PositionStatusEnum
instance SchemaType PositionStatusEnum
instance SimpleType PositionStatusEnum
 
-- | The specification of how the premium for an FX OTC option 
--   is quoted. 
data PremiumQuoteBasisEnum
instance Eq PremiumQuoteBasisEnum
instance Show PremiumQuoteBasisEnum
instance Enum PremiumQuoteBasisEnum
instance SchemaType PremiumQuoteBasisEnum
instance SimpleType PremiumQuoteBasisEnum
 
-- | Premium Type for Forward Start Equity Option 
data PremiumTypeEnum
instance Eq PremiumTypeEnum
instance Show PremiumTypeEnum
instance Enum PremiumTypeEnum
instance SchemaType PremiumTypeEnum
instance SimpleType PremiumTypeEnum
 
-- | The mode of expression of a price. 
data PriceExpressionEnum
instance Eq PriceExpressionEnum
instance Show PriceExpressionEnum
instance Enum PriceExpressionEnum
instance SchemaType PriceExpressionEnum
instance SimpleType PriceExpressionEnum
 
-- | Specifies whether the option is a call or a put. 
data PutCallEnum
instance Eq PutCallEnum
instance Show PutCallEnum
instance Enum PutCallEnum
instance SchemaType PutCallEnum
instance SimpleType PutCallEnum
 
-- | The specification of the type of quotation rate to be 
--   obtained from each cash settlement reference bank. 
data QuotationRateTypeEnum
instance Eq QuotationRateTypeEnum
instance Show QuotationRateTypeEnum
instance Enum QuotationRateTypeEnum
instance SchemaType QuotationRateTypeEnum
instance SimpleType QuotationRateTypeEnum
 
-- | The side from which perspective a value is quoted. 
data QuotationSideEnum
instance Eq QuotationSideEnum
instance Show QuotationSideEnum
instance Enum QuotationSideEnum
instance SchemaType QuotationSideEnum
instance SimpleType QuotationSideEnum
 
-- | Indicates the actual quotation style of of PointsUpFront or 
--   TradedSpread that was used to quote this trade. 
data QuotationStyleEnum
instance Eq QuotationStyleEnum
instance Show QuotationStyleEnum
instance Enum QuotationStyleEnum
instance SchemaType QuotationStyleEnum
instance SimpleType QuotationStyleEnum
 
-- | How an exchange rate is quoted. 
data QuoteBasisEnum
instance Eq QuoteBasisEnum
instance Show QuoteBasisEnum
instance Enum QuoteBasisEnum
instance SchemaType QuoteBasisEnum
instance SimpleType QuoteBasisEnum
 
-- | The specification of methods for converting rates from one 
--   basis to another. 
data RateTreatmentEnum
instance Eq RateTreatmentEnum
instance Show RateTreatmentEnum
instance Enum RateTreatmentEnum
instance SchemaType RateTreatmentEnum
instance SimpleType RateTreatmentEnum
 
-- | The contract specifies whether which price must satisfy the 
--   boundary condition. 
data RealisedVarianceMethodEnum
instance Eq RealisedVarianceMethodEnum
instance Show RealisedVarianceMethodEnum
instance Enum RealisedVarianceMethodEnum
instance SchemaType RealisedVarianceMethodEnum
instance SimpleType RealisedVarianceMethodEnum
 
-- | The specification of whether resets occur relative to the 
--   first or last day of a calculation period. 
data ResetRelativeToEnum
instance Eq ResetRelativeToEnum
instance Show ResetRelativeToEnum
instance Enum ResetRelativeToEnum
instance SchemaType ResetRelativeToEnum
instance SimpleType ResetRelativeToEnum
 
-- | The type of return associated with the equity swap. 
data ReturnTypeEnum
instance Eq ReturnTypeEnum
instance Show ReturnTypeEnum
instance Enum ReturnTypeEnum
instance SchemaType ReturnTypeEnum
instance SimpleType ReturnTypeEnum
 
-- | The convention for determining the sequence of calculation 
--   period end dates. It is used in conjunction with a 
--   specified frequency and the regular period start date of a 
--   calculation period, e.g. semi-annual IMM roll dates. 
data RollConventionEnum
instance Eq RollConventionEnum
instance Show RollConventionEnum
instance Enum RollConventionEnum
instance SchemaType RollConventionEnum
instance SimpleType RollConventionEnum
 
-- | The method of rounding a fractional number. 
data RoundingDirectionEnum
instance Eq RoundingDirectionEnum
instance Show RoundingDirectionEnum
instance Enum RoundingDirectionEnum
instance SchemaType RoundingDirectionEnum
instance SimpleType RoundingDirectionEnum
 
-- | Defines the Settlement Period Duration for an Electricity 
--   Transaction. 
data SettlementPeriodDurationEnum
instance Eq SettlementPeriodDurationEnum
instance Show SettlementPeriodDurationEnum
instance Enum SettlementPeriodDurationEnum
instance SchemaType SettlementPeriodDurationEnum
instance SimpleType SettlementPeriodDurationEnum
 
-- | Shows how the transaction is to be settled when it is 
--   exercised. 
data SettlementTypeEnum
instance Eq SettlementTypeEnum
instance Show SettlementTypeEnum
instance Enum SettlementTypeEnum
instance SchemaType SettlementTypeEnum
instance SimpleType SettlementTypeEnum
 
-- | Defines the consequences of extraordinary events relating 
--   to the underlying. 
data ShareExtraordinaryEventEnum
instance Eq ShareExtraordinaryEventEnum
instance Show ShareExtraordinaryEventEnum
instance Enum ShareExtraordinaryEventEnum
instance SchemaType ShareExtraordinaryEventEnum
instance SimpleType ShareExtraordinaryEventEnum
 
-- | The Specified Price in respect of a Transaction and a 
--   Commodity Reference Price. 
data SpecifiedPriceEnum
instance Eq SpecifiedPriceEnum
instance Show SpecifiedPriceEnum
instance Enum SpecifiedPriceEnum
instance SchemaType SpecifiedPriceEnum
instance SimpleType SpecifiedPriceEnum
 
-- | The code specification of whether a trade is settling using 
--   standard settlement instructions as well as whether it is a 
--   candidate for settlement netting. 
data StandardSettlementStyleEnum
instance Eq StandardSettlementStyleEnum
instance Show StandardSettlementStyleEnum
instance Enum StandardSettlementStyleEnum
instance SchemaType StandardSettlementStyleEnum
instance SimpleType StandardSettlementStyleEnum
 
-- | The specification of whether a percentage rate change, used 
--   to calculate a change in notional outstanding, is expressed 
--   as a percentage of the initial notional amount or the 
--   previously outstanding notional amount. 
data StepRelativeToEnum
instance Eq StepRelativeToEnum
instance Show StepRelativeToEnum
instance Enum StepRelativeToEnum
instance SchemaType StepRelativeToEnum
instance SimpleType StepRelativeToEnum
 
-- | Element to define how to deal with a none standard 
--   calculation period within a swap stream. 
data StubPeriodTypeEnum
instance Eq StubPeriodTypeEnum
instance Show StubPeriodTypeEnum
instance Enum StubPeriodTypeEnum
instance SchemaType StubPeriodTypeEnum
instance SimpleType StubPeriodTypeEnum
 
-- | The specification of how an FX OTC option strike price is 
--   quoted. 
data StrikeQuoteBasisEnum
instance Eq StrikeQuoteBasisEnum
instance Show StrikeQuoteBasisEnum
instance Enum StrikeQuoteBasisEnum
instance SchemaType StrikeQuoteBasisEnum
instance SimpleType StrikeQuoteBasisEnum
 
-- | The type of threshold. 
data ThresholdTypeEnum
instance Eq ThresholdTypeEnum
instance Show ThresholdTypeEnum
instance Enum ThresholdTypeEnum
instance SchemaType ThresholdTypeEnum
instance SimpleType ThresholdTypeEnum
 
-- | Defines points in the day when equity option exercise and 
--   valuation can occur. 
data TimeTypeEnum
instance Eq TimeTypeEnum
instance Show TimeTypeEnum
instance Enum TimeTypeEnum
instance SchemaType TimeTypeEnum
instance SimpleType TimeTypeEnum
 
-- | The time of day which would be considered for valuing the 
--   knock event. 
data TriggerTimeTypeEnum
instance Eq TriggerTimeTypeEnum
instance Show TriggerTimeTypeEnum
instance Enum TriggerTimeTypeEnum
instance SchemaType TriggerTimeTypeEnum
instance SimpleType TriggerTimeTypeEnum
 
-- | The specification of whether an option would trigger or 
--   expire depending upon whether the spot rate is above or 
--   below the barrier rate. 
data TriggerTypeEnum
instance Eq TriggerTypeEnum
instance Show TriggerTypeEnum
instance Enum TriggerTypeEnum
instance SchemaType TriggerTypeEnum
instance SimpleType TriggerTypeEnum
 
-- | The specification of, for American-style digitals, whether 
--   the trigger level must be touched or not touched. 
data TouchConditionEnum
instance Eq TouchConditionEnum
instance Show TouchConditionEnum
instance Enum TouchConditionEnum
instance SchemaType TouchConditionEnum
instance SimpleType TouchConditionEnum
 
-- | The specification of whether a payout will occur on an 
--   option depending upon whether the spot rate is above or 
--   below the trigger rate. 
data TriggerConditionEnum
instance Eq TriggerConditionEnum
instance Show TriggerConditionEnum
instance Enum TriggerConditionEnum
instance SchemaType TriggerConditionEnum
instance SimpleType TriggerConditionEnum
 
-- | The ISDA defined methodology for determining the final 
--   price of the reference obligation for purposes of cash 
--   settlement. 
data ValuationMethodEnum
instance Eq ValuationMethodEnum
instance Show ValuationMethodEnum
instance Enum ValuationMethodEnum
instance SchemaType ValuationMethodEnum
instance SimpleType ValuationMethodEnum
 
-- | The specification of a weekly roll day. 
data WeeklyRollConventionEnum
instance Eq WeeklyRollConventionEnum
instance Show WeeklyRollConventionEnum
instance Enum WeeklyRollConventionEnum
instance SchemaType WeeklyRollConventionEnum
instance SimpleType WeeklyRollConventionEnum
 
-- | The type of telephone number used to reach a contact. 
data TelephoneTypeEnum
instance Eq TelephoneTypeEnum
instance Show TelephoneTypeEnum
instance Enum TelephoneTypeEnum
instance SchemaType TelephoneTypeEnum
instance SimpleType TelephoneTypeEnum
