{-# LANGUAGE MultiParamTypeClasses, FunctionalDependencies #-}
{-# OPTIONS_GHC -fno-warn-duplicate-exports #-}
module Data.FpML.V53.Generic
  ( module Data.FpML.V53.Generic
  , module Data.FpML.V53.Shared
  , module Data.FpML.V53.Asset
  ) where
 
import Text.XML.HaXml.Schema.Schema (SchemaType(..),SimpleType(..),Extension(..),Restricts(..))
import Text.XML.HaXml.Schema.Schema as Schema
import qualified Text.XML.HaXml.Schema.PrimitiveTypes as Xsd
import {-# SOURCE #-} Data.FpML.V53.Shared
import {-# SOURCE #-} Data.FpML.V53.Asset
 
-- | A product to represent an OTC derivative transaction whose 
--   economics are not fully described using an FpML schema. 
elementGenericProduct :: XMLParser GenericProduct
elementToXMLGenericProduct :: GenericProduct -> [Content ()]
 
-- | A product to represent an OTC derivative transaction whose 
--   economics are not fully described using an FpML schema. 
elementNonSchemaProduct :: XMLParser GenericProduct
elementToXMLNonSchemaProduct :: GenericProduct -> [Content ()]
 
-- | Simple product representation providing key information 
--   about a variety of different products 
data GenericProduct
instance Eq GenericProduct
instance Show GenericProduct
instance SchemaType GenericProduct
instance Extension GenericProduct Product
 
-- | The underlying asset/index/reference price etc. whose 
--   rate/price may be observed to compute the value of the 
--   cashflow. It can be an index, fixed rate, listed security, 
--   quoted currency pair, or a reference entity (for credit 
--   derivatives). 
data TradeUnderlyer2
instance Eq TradeUnderlyer2
instance Show TradeUnderlyer2
instance SchemaType TradeUnderlyer2
 
-- | A flexible description of the type or characteristics of an 
--   option or strategy, e.g. butterfly, condor, chooser. 
data OptionType
data OptionTypeAttributes
instance Eq OptionType
instance Eq OptionTypeAttributes
instance Show OptionType
instance Show OptionTypeAttributes
instance SchemaType OptionType
instance Extension OptionType Scheme
