{-# LANGUAGE MultiParamTypeClasses, FunctionalDependencies #-}
{-# OPTIONS_GHC -fno-warn-duplicate-exports #-}
module Data.FpML.V53.Option.Bond
  ( module Data.FpML.V53.Option.Bond
  , module Data.FpML.V53.Shared.Option
  ) where
 
import Text.XML.HaXml.Schema.Schema (SchemaType(..),SimpleType(..),Extension(..),Restricts(..))
import Text.XML.HaXml.Schema.Schema as Schema
import qualified Text.XML.HaXml.Schema.PrimitiveTypes as Xsd
import {-# SOURCE #-} Data.FpML.V53.Shared.Option
 
-- | A Bond Option 
data BondOption
instance Eq BondOption
instance Show BondOption
instance SchemaType BondOption
instance Extension BondOption OptionBaseExtended
instance Extension BondOption OptionBase
instance Extension BondOption Option
instance Extension BondOption Product
 
-- | A complex type to specify the strike of a bond or 
--   convertible bond option. 
data BondOptionStrike
instance Eq BondOptionStrike
instance Show BondOptionStrike
instance SchemaType BondOptionStrike
 
-- | A complex type to specify the amount to be paid by the 
--   buyer of the option if the option is exercised prior to the 
--   Early Call Date (Typically applicable to the convertible 
--   bond options). 
data MakeWholeAmount
instance Eq MakeWholeAmount
instance Show MakeWholeAmount
instance SchemaType MakeWholeAmount
instance Extension MakeWholeAmount SwapCurveValuation
 
-- | A complex type used to specify the option and convertible 
--   bond option strike when expressed in reference to a swap 
--   curve. 
data ReferenceSwapCurve
instance Eq ReferenceSwapCurve
instance Show ReferenceSwapCurve
instance SchemaType ReferenceSwapCurve
 
-- | A complex type to specify a valuation swap curve, which is 
--   used as part of the strike construct for the bond and 
--   convertible bond options. 
data SwapCurveValuation
instance Eq SwapCurveValuation
instance Show SwapCurveValuation
instance SchemaType SwapCurveValuation
 
-- | A component describing a Bond Option product. 
elementBondOption :: XMLParser BondOption
elementToXMLBondOption :: BondOption -> [Content ()]
